@@ -715,39 +715,128 @@ accu(const T& x)
 //! accumulate values in a sparse object
 template<typename T1>
 arma_warn_unused
-arma_hot
 inline
-typename enable_if2<is_arma_sparse_type<T1>::value, typename T1::elem_type>::result
-accu(const T1& x)
+typename T1::elem_type
+accu(const SpBase<typename T1::elem_type,T1>& expr)
   {
   arma_extra_debug_sigprint();
   
   typedef typename T1::elem_type eT;
   
-  const SpProxy<T1> p(x);
+  const SpProxy<T1> P(expr.get_ref());
   
   if(SpProxy<T1>::use_iterator == false)
     {
     // direct counting
-    return arrayops::accumulate(p.get_values(), p.get_n_nonzero());
+    return arrayops::accumulate(P.get_values(), P.get_n_nonzero());
     }
   else
     {
-    typename SpProxy<T1>::const_iterator_type it     = p.begin();
-    typename SpProxy<T1>::const_iterator_type it_end = p.end();
+    typename SpProxy<T1>::const_iterator_type it = P.begin();
     
-    eT result = eT(0);
+    const uword P_n_nz = P.get_n_nonzero();
     
-    while(it != it_end)
-      {
-      result += (*it);
-      ++it;
-      }
+    eT val = eT(0);
     
-    return result;
+    for(uword i=0; i < P_n_nz; ++i)  { val += (*it); ++it; }
+    
+    return val;
     }
   }
 
 
 
+//! explicit handling of accu(A + B), where A and B are sparse matrices
+template<typename T1, typename T2>
+arma_warn_unused
+inline
+typename T1::elem_type
+accu(const SpGlue<T1,T2,spglue_plus>& expr)
+  {
+  arma_extra_debug_sigprint();
+  
+  const unwrap_spmat<T1> UA(expr.A);
+  const unwrap_spmat<T2> UB(expr.B);
+  
+  arma_debug_assert_same_size(UA.M.n_rows, UA.M.n_cols, UB.M.n_rows, UB.M.n_cols, "addition");
+  
+  return (accu(UA.M) + accu(UB.M));
+  }
+
+
+
+//! explicit handling of accu(A - B), where A and B are sparse matrices
+template<typename T1, typename T2>
+arma_warn_unused
+inline
+typename T1::elem_type
+accu(const SpGlue<T1,T2,spglue_minus>& expr)
+  {
+  arma_extra_debug_sigprint();
+  
+  const unwrap_spmat<T1> UA(expr.A);
+  const unwrap_spmat<T2> UB(expr.B);
+  
+  arma_debug_assert_same_size(UA.M.n_rows, UA.M.n_cols, UB.M.n_rows, UB.M.n_cols, "subtraction");
+  
+  return (accu(UA.M) - accu(UB.M));
+  }
+
+
+
+//! explicit handling of accu(A % B), where A and B are sparse matrices
+template<typename T1, typename T2>
+arma_warn_unused
+inline
+typename T1::elem_type
+accu(const SpGlue<T1,T2,spglue_schur>& expr)
+  {
+  arma_extra_debug_sigprint();
+  
+  typedef typename T1::elem_type eT;
+  
+  const SpProxy<T1> px(expr.A);
+  const SpProxy<T2> py(expr.B);
+  
+  typename SpProxy<T1>::const_iterator_type x_it     = px.begin();
+  typename SpProxy<T1>::const_iterator_type x_it_end = px.end();
+  
+  typename SpProxy<T2>::const_iterator_type y_it     = py.begin();
+  typename SpProxy<T2>::const_iterator_type y_it_end = py.end();
+  
+  eT acc = eT(0);
+  
+  while( (x_it != x_it_end) || (y_it != y_it_end) )
+    {
+    if(x_it == y_it)
+      {
+      acc += ((*x_it) * (*y_it));
+      
+      ++x_it;
+      ++y_it;
+      }
+    else
+      {
+      const uword x_it_col = x_it.col();
+      const uword x_it_row = x_it.row();
+      
+      const uword y_it_col = y_it.col();
+      const uword y_it_row = y_it.row();
+      
+      if((x_it_col < y_it_col) || ((x_it_col == y_it_col) && (x_it_row < y_it_row))) // if y is closer to the end
+        {
+        ++x_it;
+        }
+      else // x is closer to the end
+        {
+        ++y_it;
+        }
+      }
+    }
+  
+  return acc;
+  }
+
+
+
 //! @}
